// Maintainer: joaander
// Modified by Gang Wang
// Modified by Andrew Fiore

/*! \file Brownian.cuh
\brief Declares GPU kernel code for Brownian displacement.
*/
#include "hoomd/hoomd_config.h"
#include "ParticleData.cuh"
#include "HOOMDMath.h"
#include <cufft.h>
#include "Index1D.h"

//! Define the kernel
#ifndef __BROWNIANSAMESIZE_CUH__
#define __BROWNIANSAMESIZE_CUH__
// This only works for same size???

//! Definition for complex variable storage
#ifdef SINGLE_PRECISION
#define CUFFTCOMPLEX cufftComplex
#else
#define CUFFTCOMPLEX cufftComplex
#endif

__global__ void gpu_stokes_BrownianGenerate_kernel(Scalar4 *d_psi,
  unsigned int group_size,
  unsigned int *d_group_members,
  const unsigned int timestep,
  const unsigned int seed );

__global__ void gpu_stokes_BrownianGridGenerate_kernel(  CUFFTCOMPLEX *gridX,
  CUFFTCOMPLEX *gridY,
  CUFFTCOMPLEX *gridZ,
  Scalar4 *gridk,
  unsigned int NxNyNz,
  int Nx,
  int Ny,
  int Nz,
  const unsigned int timestep,
  const unsigned int seed,
  Scalar T,
  Scalar dt,
  Scalar quadW );

void gpu_stokes_BrownianLanczos_wrap(  Scalar4 *d_pos,
  unsigned int *d_group_members,
  unsigned int group_size,
  const BoxDim& box,
  Scalar dt,
  Scalar4 *d_vel,
  const Scalar T,
  const unsigned int timestep,
  const unsigned int seed,
  Scalar xi,
  Scalar eta,
  Scalar P,
  Scalar ewald_cut,
  Scalar ewald_dr,
  int ewald_n,
  Scalar4 *d_ewaldC1,
  Scalar4 *d_gridk,
  CUFFTCOMPLEX *d_gridX,
  CUFFTCOMPLEX *d_gridY,
  CUFFTCOMPLEX *d_gridZ,
  cufftHandle plan,
  const int Nx,
  const int Ny,
  const int Nz,
  const unsigned int *d_n_neigh,
  const unsigned int *d_nlist,
  const unsigned int *d_headlist,
  int& m_Lanczos,
  const unsigned int N_total,
  unsigned int NxNyNz,
  dim3 grid,
  dim3 threads,
  int gridBlockSize,
  int gridNBlock,
  Scalar3 gridh,
  const Scalar *d_diameter,
  Scalar cheb_error,
  Scalar self );

void gpu_stokes_CombinedMobilityBrownian_wrap( Scalar4 *d_pos,
  //Scalar4 *d_net_force,
  unsigned int *d_group_members,
  unsigned int group_size,
  const BoxDim& box,
  Scalar dt,
  Scalar4 *d_vel,
  const Scalar T,
  const unsigned int timestep,
  const unsigned int seed,
  Scalar xi,
  Scalar eta,
  Scalar P,
  Scalar ewald_cut,
  Scalar ewald_dr,
  int ewald_n,
  Scalar4 *d_ewaldC1,
  Scalar4 *d_gridk,
  CUFFTCOMPLEX *d_gridX,
  CUFFTCOMPLEX *d_gridY,
  CUFFTCOMPLEX *d_gridZ,
  cufftHandle plan,
  const int Nx,
  const int Ny,
  const int Nz,
  const unsigned int *d_n_neigh,
  const unsigned int *d_nlist,
  const unsigned int *d_headlist,
  int& m_Lanczos,
  const unsigned int N_total,
  unsigned int NxNyNz,
  dim3 grid,
  dim3 threads,
  int gridBlockSize,
  int gridNBlock,
  Scalar3 gridh,
  const Scalar *d_diameter,
  Scalar cheb_error,
  Scalar self );

#endif
