// Modified by Gang Wang
// Modified by Andrew Fiore

#include "hoomd/hoomd_config.h"
#include "HOOMDMath.h"
#include "QuaternionMath.h"
#include "ParticleData.cuh"
#include <cufft.h>

#ifndef __HELPER_CUH__
#define __HELPER_CUH__

//! Definition for comxplex variable storage
#ifdef SINGLE_PRECISION
#define CUFFTCOMPLEX cufftComplex
#else
#define CUFFTCOMPLEX cufftComplex
#endif

#include <stdio.h>
#ifdef WIN32
#include <cassert>
#else
#include <assert.h>
#endif

#define INVALID_INDEX 0xffffffff

#ifndef __ERRCHK_CUH__
#define __ERRCHK_CUH__
//! Function to check for errors
#define gpuErrchk(ans) { gpuAssert((ans), __FILE__, __LINE__); }
/*!
    \param code   returned error code
    \param file   which file the error occured in
    \param line   which line error check was tripped
    \param abort  whether to kill code upon error trigger
*/
inline void gpuAssert(cudaError_t code, const char *file, int line, bool abort=true)
{
   if (code != cudaSuccess)
   {
      fprintf(stderr,"GPUassert: %s %s %d\n", cudaGetErrorString(code), file, line);
      if (abort) exit(code);
   }
}
#endif

__global__ void gpu_stokes_DotStepOne_kernel(Scalar4 *d_a, Scalar4 *d_b, Scalar *dot_sum, unsigned int group_size, unsigned int *d_group_members);

__global__ void gpu_stokes_DotStepTwo_kernel(Scalar *dot_sum, unsigned int num_partial_sums);

__global__ void gpu_stokesrigid_ScalarMtpl_kernel(Scalar4 *d_a, Scalar4 *d_c, Scalar coeff_a, unsigned int group_size, unsigned int *d_group_members);

__global__ void gpu_stokes_LinearCombination_kernel(Scalar4 *d_a, Scalar4 *d_b, Scalar4 *d_c, Scalar coeff_a, Scalar coeff_b, unsigned int group_size, unsigned int *d_group_members);

__global__ void gpu_stokes_LinearCombination2_kernel(Scalar4 *d_a, Scalar4 *d_b, Scalar4 *d_c, Scalar4 *d_d, Scalar coeff_a, Scalar coeff_b, Scalar coeff_c, unsigned int group_size, unsigned int *d_group_members);

__global__ void gpu_stokesrigid_SetValue_kernel(Scalar4 *d_a, Scalar3 a, unsigned int group_size, unsigned int *d_group_members);

__global__ void gpu_stokesrigid_SetEqual_kernel(Scalar4 *d_a, Scalar4 *d_b, unsigned int group_size, unsigned int *d_group_members);

__global__
void gpu_stokesrigid_rigidsigma_kernel(Scalar4 *d_net_force,
				Scalar4 *d_rigidforce,
				Scalar4 *d_rigidtorque,
				unsigned int *d_rigidindex,
				Scalar4 *d_rigidorientation,
				unsigned int n_bodies,
				unsigned int nmax,
				unsigned int *d_rigidparticleindices,
				Scalar4 *d_rigidparticlepos);

__global__
void gpu_stokesrigid_rigidsigmatrans_kernel(Scalar4 *d_vel,
				Scalar4 *d_rigidvel,
				Scalar4 *d_rigidangvel,
				unsigned int *d_rigidindex,
				Scalar4 *d_rigidorientation,
				unsigned int n_bodies,
				unsigned int nmax,
				unsigned int *d_rigidparticleindices,
				Scalar4 *d_rigidparticlepos);

__global__
void gpu_stokesrigid_rigidsigmaStress_kernel(Scalar4 *d_net_force,
				Scalar3 *d_vecSii,
			        Scalar3 *d_vecSij,
				unsigned int *d_rigidindex,
				Scalar4 *d_rigidorientation,
				unsigned int n_bodies,
				unsigned int nmax,
				unsigned int *d_rigidparticleindices,
				Scalar4 *d_rigidparticlepos);

__global__ void gpu_stokesrigid_VectorSumStepOne_kernel(Scalar3 *d_a, Scalar3 *vector_sum, unsigned int group_size, unsigned int *d_group_members);

__global__ void gpu_stokesrigid_VectorSumStepOne4_kernel(Scalar4 *d_a, Scalar3 *vector_sum, unsigned int group_size, unsigned int *d_group_members);

__global__ void gpu_stokesrigid_VectorSumStepTwo_kernel(Scalar3 *vector_sum, unsigned int num_partial_sums);

__global__ void gpu_stokes_ZeroGrid_kernel(CUFFTCOMPLEX *grid, unsigned int NxNyNz);

__global__ void gpu_stokes_MatVecMultiply_kernel(Scalar4 *d_A, Scalar *d_x, Scalar4 *d_b, unsigned int group_size, int m);

__global__ void gpu_stokes_AddGrids_kernel(CUFFTCOMPLEX *d_a, CUFFTCOMPLEX *d_b, CUFFTCOMPLEX *d_c, unsigned int NxNyNz);

__global__ void gpu_stokes_ScaleGrid_kernel(CUFFTCOMPLEX *d_a, Scalar s, unsigned int NxNyNz);

__global__ void gpu_stokes_SetGridk_kernel(Scalar4 *gridk, int Nx, int Ny, int Nz, unsigned int NxNyNz, BoxDim box, Scalar xi, Scalar eta);

#endif
