// Maintainer: joaander
// Modified by Andrew Fiore

/*! \file Mobility.cuh
    \brief Declares GPU kernel code for integration considering hydrodynamic interactions on the GPU. Used by Stokes.
*/
#include "hoomd/hoomd_config.h"
#include "ParticleData.cuh"
#include "HOOMDMath.h"
#include <cufft.h>
#include "Index1D.h"
#include "TextureTools.h"

//! Define the step_one kernel
#ifndef __MOBILITYSAMESIZE_CUH__
#define __MOBILITYSAMESIZE_CUH__

//! Definition for comxplex variable storage
#ifdef SINGLE_PRECISION
#define CUFFTCOMPLEX cufftComplex
#else
#define CUFFTCOMPLEX cufftComplex
#endif

void gpu_stokes_Mobility_wrap( Scalar4 *d_pos,
                               	Scalar4 *d_vel,
                               	Scalar4 *d_net_force,
			       	unsigned int *d_group_members,
			       	unsigned int group_size,
                               	const BoxDim& box,
			       	Scalar xi,
			       	Scalar eta,
			       	Scalar ewald_cut,
			       	Scalar ewald_dr,
			       	int ewald_n,
			       	Scalar4 *d_ewaldC1,
			       	Scalar self,
			       	Scalar4 *d_gridk,
			       	CUFFTCOMPLEX *d_gridX,
			       	CUFFTCOMPLEX *d_gridY,
			       	CUFFTCOMPLEX *d_gridZ,
			       	cufftHandle plan,
			       	const int Nx,
			       	const int Ny,
			       	const int Nz,
			       	const unsigned int *d_n_neigh,
                               	const unsigned int *d_nlist,
                               	const unsigned int *d_headlist,
			       	unsigned int NxNyNz,
			       	dim3 grid,
			       	dim3 threads,
			       	int gridBlockSize,
			       	int gridNBlock,
			       	const int P,
			       	Scalar3 gridh,
			       	const Scalar *d_diameter );

__global__
void gpu_stokes_Mreal_kernel( 	Scalar4 *d_pos,
			      	Scalar4 *d_vel,
			      	Scalar4 *d_net_force,
			      	int group_size,
			      	Scalar xi,
			      	Scalar4 *d_ewaldC1,
			      	Scalar self,
			      	Scalar ewald_cut,
			      	int ewald_n,
			      	Scalar ewald_dr,
			      	unsigned int *d_group_members,
			      	BoxDim box,
			      	const unsigned int *d_n_neigh,
                              	const unsigned int *d_nlist,
                              	const unsigned int *d_headlist,
			      	const Scalar *d_diameter) ;

__global__ void gpu_stokes_Spread_kernel( 	Scalar4 *d_pos,
				    		const Scalar  *d_diameter,
				    		Scalar4 *d_net_force,
				    		CUFFTCOMPLEX *gridX,
				    		CUFFTCOMPLEX *gridY,
				    		CUFFTCOMPLEX *gridZ,
				    		int group_size,
				    		int Nx,
				    		int Ny,
				    		int Nz,
				    		unsigned int *d_group_members,
				    		BoxDim box,
				    		const int P,
				    		Scalar3 gridh,
				    		Scalar xi,
				    		Scalar eta,
						Scalar prefac,
						Scalar expfac );

__global__ void gpu_stokes_Green_kernel(CUFFTCOMPLEX *gridX, CUFFTCOMPLEX *gridY, CUFFTCOMPLEX *gridZ, Scalar4 *gridk, unsigned int NxNyNz);

__global__ void gpu_stokes_Contract_kernel( 	Scalar4 *d_pos,
				 		Scalar4 *d_vel,
				 		CUFFTCOMPLEX *gridX,
				 		CUFFTCOMPLEX *gridY,
				 		CUFFTCOMPLEX *gridZ,
				 		int group_size,
				 		int Nx,
				 		int Ny,
				 		int Nz,
				 		Scalar xi,
				 		Scalar eta,
				 		unsigned int *d_group_members,
				 		BoxDim box,
				 		const int P,
				 		Scalar3 gridh,
				 		const Scalar *d_diameter,
				 		Scalar prefac,
				 		Scalar expfac );

void gpu_stokesrigid_RigidSaddle_wrap(Scalar *x_ptr,
    Scalar *y_ptr,
    Scalar4 *d_vel,
    Scalar4 *d_pos,
    unsigned int *d_group_members,
    Scalar4 *d_ewaldC1,
    Scalar4 *d_gridk,
    CUFFTCOMPLEX *d_gridX,
    CUFFTCOMPLEX *d_gridY,
    CUFFTCOMPLEX *d_gridZ,
    const unsigned int *d_n_neigh,
    const unsigned int *d_nlist,
    const unsigned int *d_headlist,
    const Scalar *d_diameter,
    unsigned int *d_rigidindex,
    Scalar4 *d_rigidorientation,
    unsigned int *d_rigidparticleindices,
    Scalar4 *d_rigidparticlepos,
    const unsigned int group_size,
    const BoxDim box,
    Scalar xi,
    Scalar eta,
    const Scalar ewald_cut,
    const Scalar ewald_dr,
    const int ewald_n,
    Scalar self,
    cufftHandle plan,
    const int Nx,
    const int Ny,
    const int Nz,
    const unsigned int NxNyNz,
    Scalar3 gridh,
    const unsigned int N_total,
    unsigned int n_bodies,
    unsigned int nmax,
    const unsigned int N_rigid_total,
    const dim3 grid,
    const dim3 threads,
    const int gridBlockSize,
    const int gridNBlock,
    const int P,
    const dim3 grid_rigid_particle,
    const dim3 threads_rigid_particle,
    const unsigned int rigidThreadDx,
    const unsigned int rigidThreadDy);

#endif
