// Maintainer: Gang Wang
// Preconditioner for GMRES
#include "hoomd/hoomd_config.h"
#include "Helper.cuh"

#ifndef __PRECOND_CUH__
#define __PRECOND_CUH__ 

/*!
	Preconditioner helper kernel function

	d_z := (Preconditioner)^-1 * d_r
	Here, preconditioner is Sigma*Sigma^(T). (Mass and moment of inertia)
*/
__global__
void gpu_stokesrigid_InertiaInv_kernel(Scalar4 *d_r, Scalar4 *d_r_ang, Scalar4 *d_z, Scalar4 *d_z_ang, Scalar *d_rigidbodymass, Scalar4 *d_rigidmomentinertia, Scalar4 *d_rigidorientation, unsigned int group_size, unsigned int *d_group_members);

/*!
	Wrap all the functions to compute the saddle problem of rigid body motion
	Drive GPU kernel functions
*/
void gpu_stokesrigid_PreconditionerSigma_wrap(Scalar *x_ptr,
		Scalar *y_ptr,
		Scalar4 *d_temp,
		unsigned int *d_group_members,
		unsigned int *d_rigidindex,
		Scalar4 *d_rigidorientation,
		unsigned int *d_rigidparticleindices,
		Scalar4 *d_rigidparticlepos,
		Scalar *d_rigidbodymass,
		Scalar4 *d_rigidmomentinertia,
		const unsigned int group_size,
		const unsigned int N_total,
		unsigned int n_bodies,
		unsigned int nmax,
		const unsigned int N_rigid_total,
		const dim3 grid,
		const dim3 threads,
		const dim3 grid_rigid_particle,
    		const dim3 threads_rigid_particle,
		const unsigned int rigidThreadDx,
		const unsigned int rigidThreadDy,
		unsigned int grid_rigid,
		unsigned int thread_rigid,
		Scalar radius_rep);

#endif
