// Maintainer: Gang Wang
// Matrix wrappers written specific for CUSP library
// Adapted from codes of Andrew Fiore.

#include "hoomd/hoomd_config.h"
#include "PreCond.cuh"
#include <cusp/linear_operator.h>

#ifndef __PRECOND_WRAPPER_CUH__
#define __PRECOND_WRAPPER_CUH__

/*!
  cusp shell representing the preconditioner for the saddle problem (using solely Sigma matrix)
  */
class cuspPreCondSigma : public cusp::linear_operator<Scalar, cusp::device_memory>{
public:

	// Pointer members
	Scalar4 *d_temp;
	unsigned int *d_group_members;
	unsigned int *d_rigidindex;
	Scalar4 *d_rigidorientation;
	unsigned int *d_rigidparticleindices;
	Scalar4 *d_rigidparticlepos;
	Scalar *d_rigidbodymass;
	Scalar4 *d_rigidmomentinertia;

	// Variable members
	const unsigned int group_size;
	const unsigned int N_total;
	unsigned int n_bodies;
	unsigned int nmax;
	const unsigned int N_rigid_total;
	const dim3 grid;
	const dim3 threads;
	const dim3 grid_rigid_particle;
    	const dim3 threads_rigid_particle;
	const unsigned int rigidThreadDx;
	const unsigned int rigidThreadDy;
	unsigned int grid_rigid;
	unsigned int thread_rigid;
	Scalar radius_rep;

	cuspPreCondSigma(Scalar4 *d_temp,
			unsigned int *d_group_members,
			unsigned int *d_rigidindex,
			Scalar4 *d_rigidorientation,
			unsigned int *d_rigidparticleindices,
			Scalar4 *d_rigidparticlepos,
			Scalar *d_rigidbodymass,
			Scalar4 *d_rigidmomentinertia,
			const unsigned int group_size,
			const unsigned int N_total,
			unsigned int n_bodies,
			unsigned int nmax,
			const unsigned int N_rigid_total,
			const dim3 grid,
			const dim3 threads,
			const dim3 grid_rigid_particle,
    			const dim3 threads_rigid_particle,
			const unsigned int rigidThreadDx,
			const unsigned int rigidThreadDy,
			unsigned int grid_rigid,
			unsigned int thread_rigid,
			Scalar radius_rep)
				:cusp::linear_operator<Scalar,cusp::device_memory>(4*N_total+8*N_rigid_total,4*N_total+8*N_rigid_total),
				d_temp(d_temp),
				d_group_members(d_group_members),
				d_rigidindex(d_rigidindex),
				d_rigidorientation(d_rigidorientation),
				d_rigidparticleindices(d_rigidparticleindices),
				d_rigidparticlepos(d_rigidparticlepos),
				d_rigidbodymass(d_rigidbodymass),
				d_rigidmomentinertia(d_rigidmomentinertia),
				group_size(group_size),
				N_total(N_total),
				n_bodies(n_bodies),
				nmax(nmax),
				N_rigid_total(N_rigid_total),
				grid(grid),
				threads(threads),
				grid_rigid_particle(grid_rigid_particle),
    				threads_rigid_particle(threads_rigid_particle),
				rigidThreadDx(rigidThreadDx),
				rigidThreadDy(rigidThreadDy),
				grid_rigid(grid_rigid),
				thread_rigid(thread_rigid),
				radius_rep(radius_rep){}

	template<typename VectorType1, typename VectorType2>
		void operator()(VectorType1& x, VectorType2& y){

		Scalar *x_ptr = thrust::raw_pointer_cast(&x[0]);
		Scalar *y_ptr = thrust::raw_pointer_cast(&y[0]);
			
		gpu_stokesrigid_PreconditionerSigma_wrap(x_ptr,
			y_ptr,
			d_temp,
			d_group_members,
			d_rigidindex,
			d_rigidorientation,
			d_rigidparticleindices,
			d_rigidparticlepos,
			d_rigidbodymass,
			d_rigidmomentinertia,
			group_size,
			N_total,
			n_bodies,
			nmax,
			N_rigid_total,
			grid,
			threads,
			grid_rigid_particle,
    			threads_rigid_particle,
			rigidThreadDx,
			rigidThreadDy,
			grid_rigid,
			thread_rigid,
			radius_rep);
		}	
};
#endif
