// Maintainer: joaander
// Modified by Gang Wang

/*! \file StokesRigid.cuh
    \brief Declares GPU kernel code for integration considering hydrodynamic interactions on the GPU. Used by Stokes.
*/

#include "hoomd/hoomd_config.h"
#include "HOOMDMath.h"
#include "Helper.cuh"
#include "Mobility.cuh"
#include "MatrixWrapper.cuh"
#include "PreCondWrapper.cuh"
#include "Brownian.cuh"

#ifndef __STOKESRIGID_CUH__
#define __STOKESRIGID_CUH__

//! Kernel driver for the first part (no second part) of the Stokes update called by Stokes.cc
cudaError_t gpu_stokesrigid_step_one(Scalar4 *d_pos,
  Scalar4 *d_vel,
  unsigned int *d_group_members,
  unsigned int group_size,
  const BoxDim& box,
  Scalar deltaT,
  Scalar4 *d_net_force,
  const Scalar T,
  const unsigned int timestep,
  const unsigned int seed,
  Scalar xi,
  Scalar eta,
  Scalar self,
  const int P,
  Scalar3 gridh,
  Scalar ewald_cut,
  Scalar ewald_dr,
  int ewald_n,
  Scalar cheb_error,
  Scalar4 *d_ewaldC1,
  int& Lanczos,
  Scalar4 *d_gridk,
  CUFFTCOMPLEX *d_gridX,
  CUFFTCOMPLEX *d_gridY,
  CUFFTCOMPLEX *d_gridZ,
  cufftHandle plan,
  const int Nx,
  const int Ny,
  const int Nz,
  const unsigned int *d_n_neigh,
  const unsigned int *d_nlist,
  const unsigned int *d_headlist,
  const unsigned int N_total,
  const Scalar *d_diameter,
  Scalar4 *d_rigidvel,
  Scalar4 *d_rigidangvel,
  //Scalar4 *d_rigidforce,
  //Scalar4 *d_rigidtorque,
  unsigned int *d_rigidindex,
  Scalar4 *d_rigidorientation,
  unsigned int n_bodies,
  unsigned int nmax,
  unsigned int *d_rigidparticleindices,
  Scalar4 *d_rigidparticlepos,
  Scalar4 *d_rigidcom,
  int3 *d_rigidimage,
  unsigned int rigidThreadDx,
  unsigned int rigidThreadDy,
  unsigned int rigidGrid,
  unsigned int N_rigid_total,
  Scalar *d_rigidbodymass,
  Scalar4 *d_rigidmomentinertia,
  int3 *d_image,
  bool hydrostress_now,
  Scalar3 *d_vecSii,
  Scalar3 *d_vecSij,
  Scalar3 *SBii_total,
  Scalar3 *SBij_total,
  Scalar3 *SPii_total,
  Scalar3 *SPij_total,
  Scalar *d_net_virial,
  unsigned int virial_pitch,
  Scalar shear_rate,
  Scalar3 *test);
#endif
