from hoomd_script import *
from hoomd_plugins import pse_rigidv0
import unittest
import os
import numpy
import math

# unit tests for basic object creation and parameter set
class basic_tests (unittest.TestCase):
    def setUp(self):
        print
        system = init.create_random(N=8000, phi_p=0.1)
        for p in system.particles:
            p.body = 0
        system.sysdef.getRigidData().initializeData()

    # tests basic creation of the integration method
    def test_basic_run(self):
        all = group.all()
        integrate.mode_standard(dt = 0.005)
        pse_rigidv0.integrate.pse_rigid(group = all, T = 0)
        run(10)
        del all

    # test set_params
    def test_set_params(self):
        all = group.all()
        integrate.mode_standard(dt = 0.005)
        bd = pse_rigidv0.integrate.pse_rigid(group = all, T = 0)
        bd.set_params(T = 1.0, delimiter = 'a', function_form = pse_rigidv0.shear_function.steady(dt = 1e-4, shear_rate = 1))
        run(10)
        bd.disable()
        del bd, all

    # test the variant
    def test_variant_basic(self):
        all = group.all()
        dt = 0.005
        integrate.mode_standard(dt = dt)
        bd = pse_rigidv0.integrate.pse_rigid(group = all, T = 0, function_form = pse_rigidv0.shear_function.sine(dt = 1e-4, shear_rate = 1, shear_freq = 1))
        box_resize = update.box_resize(xy = pse_rigidv0.variant.shear_variant(function_form = pse_rigidv0.shear_function.steady(dt = dt), total_timestep = 100))
        box_resize.set_params(scale_particles = False)
        run(10)
        bd.stop_shear()
        del all, box_resize, dt, bd

    def tearDown(self):
        init.reset()

if __name__ == '__main__':
    unittest.main(argv = ['test.py', '-v'])
