#define BOOST_TEST_MODULE ShearFunction
#include "boost_utf_configure.h"

#include <iostream>
#include <boost/shared_ptr.hpp>

#include "ShearFunction.h"

// Test cases for SteadyShearFunction class

BOOST_AUTO_TEST_CASE( steady_shear_basic )
{
    const boost::shared_ptr<ShearFunction> steady( new SteadyShearFunction(0.1, 100, 0.01) );
    MY_BOOST_CHECK_CLOSE(steady->getShearRate(300), 0.1, tol_small);
    MY_BOOST_CHECK_SMALL((Scalar)steady->getStrain(100), tol_small); // strain = 0
    MY_BOOST_CHECK_CLOSE(steady->getStrain(300), 0.2, tol_small); // strain > 0
    BOOST_CHECK_EQUAL_UINT(steady->getOffset(), 100);
}

BOOST_AUTO_TEST_CASE( steady_shear_large )
{
    const boost::shared_ptr<ShearFunction> steady( new SteadyShearFunction(5, 100, 0.01) );
    MY_BOOST_CHECK_CLOSE(steady->getShearRate(30000), 5, tol_small);
    MY_BOOST_CHECK_CLOSE(steady->getStrain(1100), 50, tol_small); // strain > 1
    BOOST_CHECK_EQUAL_UINT(steady->getOffset(), 100);
}

// Test cases for SinShearFunction class

BOOST_AUTO_TEST_CASE( sin_shear_basic )
{
    const boost::shared_ptr<ShearFunction> sinusoidal( new SinShearFunction(0.1, 0.5, 20000, 0.005) );
    MY_BOOST_CHECK_CLOSE(sinusoidal->getShearRate(30000), 0.1, tol); // shear rate = max
    MY_BOOST_CHECK_SMALL((Scalar)sinusoidal->getShearRate(30500), tol_small); // shear rate ~ 0
    MY_BOOST_CHECK_CLOSE(sinusoidal->getShearRate(30050), 0.07071068, tol); // shear rate > 0
    MY_BOOST_CHECK_SMALL((Scalar)sinusoidal->getStrain(40000), tol_small); // strain ~ 0
    MY_BOOST_CHECK_CLOSE(sinusoidal->getStrain(30100), 0.03183099, tol); // strain > 0
    BOOST_CHECK_EQUAL_UINT(sinusoidal->getOffset(), 20000);
}

BOOST_AUTO_TEST_CASE( sin_shear_negative )
{
    const boost::shared_ptr<ShearFunction> sinusoidal( new SinShearFunction(30, 10, 3, 0.01) );
    MY_BOOST_CHECK_CLOSE(sinusoidal->getShearRate(3), 30, tol); // shear rate = max
    MY_BOOST_CHECK_CLOSE(sinusoidal->getShearRate(8), -30, tol); // shear rate = -max
    MY_BOOST_CHECK_SMALL((Scalar)sinusoidal->getStrain(8), tol_small); // strain ~ 0
    MY_BOOST_CHECK_CLOSE(sinusoidal->getStrain(10), -0.45409604, tol); // strain < 0
    BOOST_CHECK_EQUAL_UINT(sinusoidal->getOffset(), 3);
}

BOOST_AUTO_TEST_CASE( sin_shear_large )
{
    const boost::shared_ptr<ShearFunction> sinusoidal( new SinShearFunction(30, 0.1, 0, 0.02) );
    MY_BOOST_CHECK_CLOSE(sinusoidal->getShearRate(5000), 30, tol); // shear rate = max
    MY_BOOST_CHECK_CLOSE(sinusoidal->getShearRate(250), -30, tol); // shear rate = -max
    MY_BOOST_CHECK_CLOSE(sinusoidal->getStrain(125), 47.746483, tol); // strain > 1
    MY_BOOST_CHECK_CLOSE(sinusoidal->getStrain(375), -47.746483, tol); // strain < -1
    BOOST_CHECK_EQUAL_UINT(sinusoidal->getOffset(), 0);
}
