#define BOOST_TEST_MODULE VariantShearFunction
#include "boost_utf_configure.h"

#include <iostream>
#include <boost/shared_ptr.hpp>

#include "VariantShearFunction.h"

// Test cases for VariantShearFunction.wrapValue

BOOST_AUTO_TEST_CASE( variant_shear_wrap )
{
    const boost::shared_ptr<ShearFunction> dummyFun( new SteadyShearFunction(0.1, 100, 0.01) );
    VariantShearFunction variant1(dummyFun, 100, 0, 1); // classic case [0,1]
    MY_BOOST_CHECK_CLOSE(variant1.wrapValue(0.5), 0.5, tol); // functionValue in (min, max)
    MY_BOOST_CHECK_CLOSE(variant1.wrapValue(1.3), 0.3, tol); // functionValue > max
    MY_BOOST_CHECK_CLOSE(variant1.wrapValue(100.78), 0.78, tol); // functionValue > max
    MY_BOOST_CHECK_CLOSE(variant1.wrapValue(-0.1), 0.9, tol); // functionValue < min
    MY_BOOST_CHECK_CLOSE(variant1.wrapValue(-7.001), 0.999, tol); // functionValue < min

    VariantShearFunction variant2(dummyFun, 100, -0.2, 0.8); // [-0.2, 0.8]
    MY_BOOST_CHECK_CLOSE(variant2.wrapValue(-0.1), -0.1, tol); // functionValue in (min, max)
    MY_BOOST_CHECK_CLOSE(variant2.wrapValue(0.9), -0.1, tol); // functionValue > max
    MY_BOOST_CHECK_CLOSE(variant2.wrapValue(0.8), -0.2, tol); // functionValue = max
    MY_BOOST_CHECK_CLOSE(variant2.wrapValue(-0.2), -0.2, tol); // functionValue = min
    MY_BOOST_CHECK_CLOSE(variant2.wrapValue(-3), 0, tol); // functionValue < min

    VariantShearFunction variant3(dummyFun, 100, -0.3, 1.1); // [-0.3, 1.1], will not happen in real application
    MY_BOOST_CHECK_CLOSE(variant3.wrapValue(1), 1, tol); // functionValue in (min, max)
    MY_BOOST_CHECK_CLOSE(variant3.wrapValue(1.2), -0.2, tol); // functionValue > max
    MY_BOOST_CHECK_CLOSE(variant3.wrapValue(-10), -0.2, tol); // functionValue < min
}

// Test cases for VariantShearFunction.getValue

BOOST_AUTO_TEST_CASE( variant_shear_get_steady )
{
    const boost::shared_ptr<ShearFunction> steady( new SteadyShearFunction(1, 100, 0.01) );
    VariantShearFunction variantSteady(steady, 930, -0.5, 0.5);
    MY_BOOST_CHECK_SMALL((Scalar)variantSteady.getValue(30), tol_small); // timestep < offset
    MY_BOOST_CHECK_SMALL((Scalar)variantSteady.getValue(100), tol_small); // timestep < offset
    MY_BOOST_CHECK_CLOSE(variantSteady.getValue(1040), 0.3, tol); // timestep > offset + totalstep
    MY_BOOST_CHECK_CLOSE(variantSteady.getValue(1030), 0.3, tol); // timestep = offset + totalstep
    MY_BOOST_CHECK_CLOSE(variantSteady.getValue(170), -0.3, tol); // timestep within (offset, offset + totalstep) -> negative
    MY_BOOST_CHECK_CLOSE(variantSteady.getValue(245), 0.45, tol); // timestep within (offset, offset + totalstep) -> positive
}

BOOST_AUTO_TEST_CASE( variant_shear_get_oscillatory )
{
    const boost::shared_ptr<ShearFunction> sinusoidal( new SinShearFunction(3, 1, 50, 0.01) );
    VariantShearFunction variantSin(sinusoidal, 870, -1, 1);
    MY_BOOST_CHECK_SMALL((Scalar)variantSin.getValue(45), tol_small); // timestep < offset
    MY_BOOST_CHECK_CLOSE(variantSin.getValue(950), -0.45409604, tol); // timestep > offset + totalstep
    MY_BOOST_CHECK_CLOSE(variantSin.getValue(175), 0.47746483, tol); // timestep within range
}

BOOST_AUTO_TEST_CASE( variant_shear_get_laos )
{
    const boost::shared_ptr<ShearFunction> sinusoidal( new SinShearFunction(3, 0.1, 0, 0.01) );
    VariantShearFunction variantSin(sinusoidal, 1000, -0.5, 0.5);
    MY_BOOST_CHECK_CLOSE(variantSin.getValue(125), 0.37618619, tol);
    MY_BOOST_CHECK_CLOSE(variantSin.getValue(250), -0.22535171, tol);
    MY_BOOST_CHECK_CLOSE(variantSin.getValue(10), 0.29980265, tol);
}
